setwd('/home/allgoodguys/Documents/Studying/Lund_PhD/epistles/04_article_nat-actor/analysis')

library(rstan); library(shinystan); library (brms)
library (randomForest)
source("~/Documents/Studying/Lund_PhD/epistles/02_article_triplets/triplets_analysis/source_for_multiple_cores.R")

acoustics = read.csv ('278_july2016.csv')
# aggregate (HNR_median~corpus, acoustics, mean)
# aggregate (duration~(corpus=='ut'), acoustics, mean)
# aggregate (duration~(corpus=='ut'), acoustics, sd)
# summary(lm(duration~(corpus=='ut'), acoustics))
# t.test (acoustics$duration[acoustics$corpus=='ut'], acoustics$duration[acoustics$corpus!='ut'])

answers = read.csv ('real-fake_indiv-answers.csv')
answers$sound = as.character (answers$sound)
answers$sound = substr (answers$sound, 1, nchar(answers$sound)-4)

df = answers[,c('id','sound','response')]
df$row = match (df$sound, acoustics$name278)

table (df$sound[is.na(df$row)]) # pleasure_sex files renamed

df$sound=as.character(df$sound)
acoustics$file=as.character(acoustics$file)
df$row = ifelse (is.na(df$row), 
                 match ( substr (df$sound,nchar(df$sound)-10,nchar(df$sound)),
                         substr (acoustics$file,nchar(acoustics$file)-10,nchar(acoustics$file)) ), 
                         df$row)

# check:
# df$file = acoustics$file[df$row]
# sum(is.na(df$file))
# df = df[,1:ncol(df)-1]

vars = c("pitch_manually", "pitch_sd", "HNR_median","HNR_sd", "quartile25_median", "quartile25_sd", "harmonics_median",  "specSlope_median", "specSlope_sd", "syllableLength_median", "syllableLength_sd", "interBurst_median", "interBurst_sd", "ampl_voiced_median", "ampl_voiced_sd", "voiced")
# with duration: vars = c("pitch_manually", "pitch_sd", "HNR_median","HNR_sd", "quartile25_median", "quartile25_sd", "harmonics_median",  "specSlope_median", "specSlope_sd", "syllableLength_median", "syllableLength_sd", "interBurst_median", "interBurst_sd", "ampl_voiced_median", "ampl_voiced_sd", "voiced", "duration")

for (v in 1:length(vars)){
  df = cbind (df, acoustics[df$row,vars[v]])
}
colnames(df)[5:ncol(df)] = vars
# write.csv (df, 'real-fake_indiv-answers_with-acoustics.csv')

# duration only
mod_dur = brm (response~duration+(1|sound)+(1|id), df, family='bernoulli', warmup=100, iter=200, chains=4, cores=4, ranef=F)
a=as.matrix(mod_dur)
colnames(a)
quantile(a[,2],probs=c(.5,.025,.975))

## load data
df = read.csv ('real-fake_indiv-answers_with-acoustics.csv')[,-1]


rescale_df = function(df){
  df$pitch_manually = ifelse (!is.na(df$pitch_manually), log2(df$pitch_manually), NA)
  df$pitch_sd = log2(df$pitch_sd+1) # b/c it's occasionally 0
  df$quartile25_median = log2(df$quartile25_median+1)
  df$quartile25_sd = log2(df$quartile25_sd+1)
  return (df)
}

df = rescale_df (df)
df[,5:ncol(df)] = scale (df[,5:ncol(df)])
df[is.na(df)] <- 0
# summary(df)

# a quick overview
layout (matrix(1:16, ncol=4))
for (v in vars){
  boxplot (get(v)~response, df, main=v)
}
layout (matrix(1))

## STAN stuff
modelString = "
data { 
  int<lower=1> N;  # number of observations 
  int Y[N];  # response variable 
  int<lower=1> K;  # number of fixed effects 
  matrix[N, K] X;  # FE design matrix 
  # data for random effects of id 
  int<lower=1> J_1[N];  # RE levels 
  int<lower=1> N_1;  # number of levels 
  int<lower=1> K_1;  # number of REs 
  real Z_1[N];  # RE design matrix 
  # data for random effects of sound 
  int<lower=1> J_2[N];  # RE levels 
  int<lower=1> N_2;  # number of levels 
  int<lower=1> K_2;  # number of REs 
  real Z_2[N];  # RE design matrix 
} 
transformed data { 
} 
parameters { 
  real b_Intercept;  # fixed effects Intercept 
  vector[K] b;  # fixed effects 
  real<lower=0> sigmaX; # SD of betas
  vector[N_1] pre_1;  # unscaled REs 
  real<lower=0> sd_1;  # RE standard deviation 
  vector[N_2] pre_2;  # unscaled REs 
  real<lower=0.01> sd_2;  # RE standard deviation 
} 
transformed parameters { 
  vector[N] eta;  # linear predictor 
  vector[N_1] r_1;  # REs 
  vector[N_2] r_2;  # REs 
  # compute linear predictor 
  eta <- X * b + b_Intercept; 
  r_1 <- sd_1 * (pre_1);  # scale REs 
  r_2 <- sd_2 * (pre_2);  # scale REs 
  # if available add REs to linear predictor 
  for (n in 1:N) { 
    eta[n] <- eta[n] + Z_1[n] * r_1[J_1[n]] + Z_2[n] * r_2[J_2[n]]; 
  } 
} 
model { 
  # prior specifications 
  sd_1 ~ cauchy(0, 5); 
  pre_1 ~ normal(0, 1); 
  sd_2 ~ cauchy(0, 5); 
  pre_2 ~ normal(0, 1); 
  sigmaX ~ normal (0, 10);
  b ~ cauchy(0, sigmaX);
  # likelihood contribution 
  Y ~ bernoulli_logit(eta); 
} 
"
stanmod <- stan_model(model_code = modelString, model_name = "stanmod") 

formula = paste0('response ~ ',paste(vars,collapse='+'), '+ (1|id) + (1|sound)')
dataList = make_standata (as.formula(formula) , data=df, family='bernoulli')

fit <- sampling(stanmod, data = dataList, pars = c( "b_Intercept" ,  "b", 'sigmaX' ), chains = 4, iter = 500, thin = 1, cores=4)
# saveRDS(fit, 'mod_indiv-resp~acoustics.RDS')  # fit = readRDS('mod_indiv-resp~acoustics.RDS')

stan_trace(fit)
a = as.matrix(fit) # for duration: quantile(a[,17],probs=c(.5,.025,.975))
b = stan_plot (fit, pars = colnames(a)[2:(ncol(a)-2)])
# b$scales$scales[[2]]$labels = vars
b$scales$scales[[2]]$labels = c('Pitch median','Pitch SD','HNR median','HNR SD','First quartile median','First quartile SD','Energy above F0 median','Spectral slope median','Spectral slope SD', 'Syllable median','Syllable SD','Interburst median','Interburst SD','Amplitude median','Amplitude SD','Voiced')  # NB: set the labels in ggplot object to the variable names, otherwise you get b[1], b[2] etc. Clever, no?  :))
b + geom_vline(xintercept=0, linetype=2)  # + ggtitle("All emotions")
# ggsave ('/home/allgoodguys/Documents/Studying/Lund_PhD/epistles/04_article_nat-actor/pix/betas_human-judgments.jpg', width=8, height=10, scale=2, units='cm', dpi=300)
# ggsave ('/home/allgoodguys/Documents/Studying/Lund_PhD/epistles/04_article_nat-actor/pix/betas_human-judgments.tiff', width=8, height=10, scale=2, units='cm', dpi=600)


# check model accuracy:
colnames(a)=c('int',vars,'sigmaX','lp__')
coefs = apply (a, 2, median)
for (i in 1:nrow(df)){
  logit = coefs[1] + sum(coefs[vars]*df[i,vars])
  df$authenticity[i] = 1/(1+exp(-logit))
}
df$pred = ifelse (df$authenticity>.5, 'real','fake')
df$guessed = (df$pred==df$response)
mean(df$guessed)
aggregate (guessed~response, df, mean)

mod3 = randomForest (response ~ pitch_manually+pitch_sd+HNR_median+HNR_sd+quartile25_median+quartile25_sd+harmonics_median+specSlope_median+specSlope_sd+syllableLength_median+syllableLength_sd+interBurst_median+interBurst_sd+ampl_voiced_median+ampl_voiced_sd+voiced, df, importance=TRUE, ntree=1000, strata=df$response, sampsize=rep(min(table(df$response)), length(unique(df$response)))) # + duration
mod3 # 68% correct w/o duration, 67 % with duration
varImpPlot(mod3, type='1')



## extras


## for laughs only
df$emotion = acoustics$emotion[match(df$sound, acoustics$file)]
laugh = df[df$emotion=='amusement',]
dataList = make_standata (as.formula(formula) , data=laugh, family='bernoulli')

summary(glm(response ~ pitch_manually+pitch_sd+HNR_median+HNR_sd+bandLwr_median+bandLwr_sd+harmonics_median+harmonics_sd+syllableLength_median+syllableLength_sd+interBurst_median+interBurst_sd+ampl_RMS_median+ampl_RMS_sd+voiced, laugh, family='binomial'))

layout (matrix(1:16, ncol=4))
for (v in vars){
  boxplot (get(v)~response, laugh, main=v)
}
layout (matrix(1))

library(lme4)
coefs = matrix(nrow=length(vars), ncol=3)
for (v in 1:length(vars)){
  coefs[v,1] = vars[v]
  coefs[v, 2:3] = as.numeric ( summary(glmer(laugh$response~laugh[,vars[v]]+(1|laugh$sound)+(1|laugh$id), family='binomial'))$coef[2,c(1,4)] )
}
colnames(coefs) = c('predictor', 'beta', 'p')
coefs[,3] = round (as.numeric(coefs[,3]) * 16, 4) # Bonferroni


### aggregate table of responses
df$real = ifelse (df$response=='real', T, F)
df$ut = ifelse (substr(as.character(df$sound),1,2)=='ut', T, F)
a = as.data.frame (aggregate (real~sound, df, mean))
a = cbind (a, df[match (a$sound, df$sound), 5:19])
a$ut = ifelse (substr(as.character(a$sound),1,2)=='ut', T, F)

mod4 = randomForest (real ~ pitch_manually+pitch_sd+HNR_median+HNR_sd+bandLwr_median+bandLwr_sd+harmonics_median+harmonics_sd+syllableLength_median+syllableLength_sd+interBurst_median+interBurst_sd+ampl_RMS_median+ampl_RMS_sd+voiced, a, importance=TRUE, ntree=1000)
mod4 # 52% var correct 
varImpPlot(mod4, type='1') # pitch, harmonics_sd, pitch_sd, HNR_median, bandLwr_median

hist (a$real, freq=F); lines(density(a$real))
hist (a$real[a$ut==T], freq=F, xlim=c(0,1)); lines(density(a$real[a$ut==T]))
hist (a$real[a$ut==F], freq=F, xlim=c(0,1)); lines(density(a$real[a$ut==F]))

plot (density(a$real[a$ut==T]), xlim=c(0,1))
lines(density(a$real[a$ut==F]))

mod = brm (real ~ pitch_manually+pitch_sd+HNR_median+HNR_sd+bandLwr_median+bandLwr_sd+harmonics_median+harmonics_sd+syllableLength_median+syllableLength_sd+interBurst_median+interBurst_sd+ampl_RMS_median+ampl_RMS_sd+voiced, data=a, family='gaussian')
plot(mod)
summary(mod)
# essentially the same as above but less statistically justified




## controlling for gender
library (lme4)
df$emotion = acoustics[df$row, 'emotion']
df$sex = acoustics[df$row, 'sex']
fear = df[df$emotion=='fear',]
summary(glm(response ~ pitch_manually*sex, fear, family='binomial'))


