setwd ('/home/allgoodguys/Documents/Studying/Lund_PhD/experiments/real-fake/analysis')

## extraction
folder = "/home/allgoodguys/Documents/Studying/Lund_PhD/experiments/real-fake/output"  
filenames <- list.files(folder, pattern="*.csv", full.names=TRUE)  # open all
int <- lapply(filenames, read.csv, header=F, stringsAsFactors=FALSE)
for (i in 1:length(int)){ # to number responses for each participant
  int[[i]]$trialNo = 1:nrow(int[[i]])
}
df = do.call(rbind, int)[,3:13]
colnames(df) = c('id','lang','showEmotion','sound','emotion','response','correct','reps','time','timeTotal','trialNo')

df$showEmotion = ifelse (df$showEmotion==1, T, F)
df$corpus = apply (as.matrix(df$sound), 1, function(x)unlist(strsplit(as.character(x), '_'))[1])
df = df[df$sound!='undefined',]

# choose cutoff value for min number of trials per participant
temp = rep(NA, 150)
for (i in 1:150){
  temp[i] = nrow(droplevels (df[!df$id %in% names(table(df$id)[table(df$id)<i]), ]))
}
plot (1:150, temp, type='l')

df = droplevels (df[!df$id %in% names(table(df$id)[table(df$id)<100]), ]) # drop participants who did <100 sounds
# t = as.matrix (table(df$id))
# length(t[t>99])
# length(t[t <= 99])
short = droplevels (df[!df$id %in% names(table(df$id)[table(df$id)<10]), ]) # drop participants who did <100 sounds
ts = as.matrix(table(short$id))
summary(ts)
length(ts[ts>10])

# translate back into English
ems_en = c('achievement','amusement','anger','disgust','fear','pain','pleasure','sadness')
ems_ru = c('ликование','веселье','злость','отвращение','страх','боль','удовольствие','грусть')
ems_sv = c('jubel','munterhet','ilska','äckel','rädsla','smärta','njutning','sorg')
ems_pt = c('proeza/triunfo','divertimento','raiva','nojo','medo','dor','prazer','tristeza')
ems_fr = c('réussite','amusement/joie', 'colère', 'dégoût', 'peur','douleur', 'plaisir', 'tristesse')
for (i in 1:8) {
  df$emotion = ifelse(df$emotion==ems_ru[i] | df$emotion==ems_sv[i] | df$emotion==ems_pt[i] | df$emotion==ems_fr[i],ems_en[i],df$emotion)
}
table (df$emotion)
df$ut = ifelse (df$corpus=='ut',T,F)
df$real = ifelse (df$response=='real',T,F)
write.csv (df, 'temp.csv'); df = read.csv('temp.csv')[,-1]

df$corpus = factor(df$corpus, levels=sort(levels(df$corpus)))

# with / without noise
df$noisy = ifelse(!is.na(df$lang) & substr(df$lang,1,5)=='noisy', T, F)
df$lang=as.character(df$lang); df$lang = ifelse (df$noisy==T, substr(df$lang,7,nchar(df$lang)), df$lang); df$lang=as.factor(df$lang)

summary(df$noisy)
mean (df$correct[df$noisy==T])
hist ( aggregate(correct~id, df[df$noisy==T & !df$id %in% names(table(df$id)[table(df$id)<100]), ], mean)[,2] )
# table(droplevels(df$id[df$noisy==T]))


# standardize language codes
# levels(df$lang)
df$lang = as.character(df$lang)
df$lang = ifelse(df$lang=='english' | df$lang=='eNGLISH', 'English', df$lang)
df$lang = ifelse(df$lang=='french' | df$lang=='fr', 'French', df$lang)
df$lang = ifelse(df$lang=='portuguese' | df$lang=='pt', 'Portuguese', df$lang)
df$lang = ifelse(df$lang=='', 'null', df$lang)
df$lang = as.factor(df$lang)
# levels(df$lang)
# table (df$lang)

# aggregate (correct~lang, df, mean)
# aggregate (correct~lang=='en', df, mean)
# aggregate (real~(lang=='en')+ut, df, mean)
# aggregate (real~(lang=='Portuguese')+(corpus=='lima'), df, mean)

df$langCaller = ifelse (df$corpus=='ut' | df$corpus=='cordaro' | df$corpus=='simon', 'English', 'Other')
df$langCaller = ifelse (df$corpus=='hawk', 'Dutch', df$langCaller)
df$langCaller = ifelse (df$corpus=='lima', 'Portuguese', df$langCaller)
df$langCaller = ifelse (df$corpus=='belin' | df$corpus=='maurage', 'French', df$langCaller)
df$langCaller = as.factor(df$langCaller)
table (df$langCaller)

df$langRater = ifelse(df$lang=='English' | df$lang=='Portuguese' | df$lang=='French' | df$lang=='Dutch', as.character(df$lang), 'Other')
df$langRater = as.factor (df$langRater)
levels(df$langRater)
table (df$langRater) / 278 # no. of ratings per sound in each language group
aggregate (id~langRater, df, function(x)length(unique(x))) # no. of participants in each language group (only those who did >50 trials)

df$langSame = ifelse (as.character(df$langCaller)==as.character(df$langRater),T,F)

# write.csv (df, 'real-fake_indiv-answers.csv')

## Filter to only with / without noise
df_copy = df # write.csv(df_copy, 'real-fake_indiv-answers.csv')
# df = droplevels (df_copy[df_copy$noisy==T,])  # noisy only 
# df = droplevels (df_copy[df_copy$noisy!=T,])  # w/o noise only 


## descriptives
mean (table(df$sound)) # ratings per sound
min (table(df$sound)); max(table(df$sound))  # table(df$sound)[table(df$sound)==0]
hist(table(df$sound))
length(unique(df$id)); mean(table(df$id)); sd(table(df$id))

mean (df$correct)



# average time per person
temp = droplevels (df[!df$id %in% names(table(df$id)[table(df$id)<100]), ])
a = aggregate (timeTotal~id, temp, max)[,2]
mean(a)/60000 # mean completion time, in min (13 min)
median(a)/60000 # 11 min


# see if the two parts of experiment have ~equal numbers of participants
temp = aggregate (id~sound, df, function(x)length(unique(x)))
hist(temp$id, breaks=30) 

temp1 = aggregate (id~sound, df[df$langRater=='English' | df$langRater=='Other',], function(x)length(unique(x)))
hist(temp1$id, breaks=30)  

temp2 = aggregate (id~sound, df[df$langRater=='French' | df$langRater=='Portuguese',], function(x)length(unique(x)))
hist(temp2$id, breaks=30)  
