setwd ('/home/allgoodguys/Documents/Studying/Lund_PhD/experiments/real-fake/analysis')
require(brms)
require(shinystan)
require(ggplot2)

df_copy = read.csv('real-fake_indiv-answers.csv')
# df = droplevels (df_copy[df_copy$noisy==T,])  # noisy only 
# df = droplevels (df_copy[df_copy$noisy!=T,])  # w/o noise only 

## Are some emotions >likely to be rated as real when the emotion is shown vs not shown?
aggregate (correct~showEmotion, df, mean)
diff(aggregate (correct~showEmotion, df, mean)[,2])

a1 = aggregate (correct~showEmotion+emotion, df[df$ut==T,], mean)
a2 = aggregate (correct~showEmotion+emotion, df[df$ut==F,], mean)

ggplot (a1, aes(x=emotion, y=correct, fill=showEmotion)) +
  geom_bar(stat='identity', position=position_dodge()) + 
  geom_hline(yintercept=.5, linetype=2) + ggtitle('YouTube') +
  scale_fill_manual(values=c('gray75','gray40'), name='Show emotion', labels=c('No','Yes')) +
  xlab("") + ylab('Probability of responding REAL, %') + theme_bw ()

ggplot (a2, aes(x=emotion, y=correct, fill=showEmotion)) +
  geom_bar(stat='identity', position=position_dodge()) + 
  geom_hline(yintercept=.5, linetype=2) + ggtitle('Play-acted corpora') +
  scale_fill_manual(values=c('gray75','gray40'), name='Show emotion', labels=c('No','Yes')) +
  xlab("") + ylab('Probability of responding REAL, %') + theme_bw ()


a = aggregate (correct~showEmotion+ut+emotion, df, mean)
df_plot = data.frame (ut = rep(c(FALSE,TRUE),8), emotion=rep(levels(df$emotion),each=2))
s = seq(1, (nrow(a)-1), by=2)
for (i in 1:length(s)){
  rowNum = s[i]
  df_plot$delta[i] = (a$correct[rowNum+1]-a$correct[rowNum]) * 100
}

df_plot$delta = as.numeric (df_plot$delta)

ggplot (df_plot, aes(x=emotion, y=delta, fill=ut)) +
  geom_bar(stat='identity', position=position_dodge()) + 
  scale_fill_manual(values=c('gray75','gray40'), name='Corpus', labels=c('Actor','YouTube')) +
  xlab("") + ylab('More REAL when emotion known, %') + theme_bw ()


mod0 = glm (correct~showEmotion + emotion*ut, df, family='binomial')
summary(mod0)
drop1 (mod0, test='Chisq')
# need interaction between condition and corpus/emotion?
drop1 (glm (correct~showEmotion*corpus, df, family='binomial'), test='Chisq')
drop1 (glm (correct~showEmotion*emotion, df, family='binomial'), test='Chisq')


mod = brm (correct~showEmotion+emotion*ut + (1|sound) + (1|id), data = df, n.warmup=100, n.iter=300, n.chains=4, n.cluster=4, family='bernoulli', ranef=F)  
a = summary(mod)  
exp(a$fixed[2,c(1,3,4)])
# stancode (modBRMS_cat)
# saveRDS(modBRMS_cat, 'temp.RDS')
