setwd ('/home/allgoodguys/Documents/Studying/Lund_PhD/experiments/real-fake/analysis')
require(brms)
require(shinystan)
require(ggplot2)
library(grid)
library(pBrackets) 

df_copy = read.csv('real-fake_indiv-answers.csv')
# df = droplevels (df_copy[df_copy$noisy==T,])  # masking noise
# df = droplevels (df_copy[df_copy$noisy!=T,])  # original

# overall accuracy
mean (df$correct)
mod00 = brm(correct ~ 1 + (1|sound) + (1|id), data=df, n.warmup=100, n.iter=500, n.chains=4, n.cluster=4, family='bernoulli', ranef=F)
plot(mod00)
a = summary(mod00)
b = a$fixed[1,c(1,3,4)]
exp(b) # odds


## Are YouTube sounds more likely to be rated as "real"?
aggregate (real~ut, df, mean)


mod0 = brm (real~ut + (1|sound) + (1|id), data = df, n.warmup=100, n.iter=500, n.chains=4, n.cluster=4, family='bernoulli', ranef=F)
# saveRDS(mod0, 'mod0_corpus.RDS')  # mod0 = readRDS('mod0_corpus.RDS')
a = summary(mod0)
b = a$fixed[2,c(1,3,4)]
exp(b) # odds

coda0 = posterior_samples(mod0)
colnames(coda0)
fake = 1 / (1+exp(-coda0[,1]))
ut = 1 / (1+exp(-(coda0[,1]+coda0[,2])))
quantile (ut-fake, c(.5, .025, .975)) # delta


## Which corpora are >likely to be rated as real?
aggregate (real~corpus, df, mean)
aggregate (real~showEmotion+corpus, df, mean)

temp = aggregate (real~corpus+sound, df, mean)
# temp = aggregate (real~corpus+id, df, mean)  # to show variation by sound or by id. Note var is much greater per id
boxplot (real~corpus, temp)
ggplot (temp, aes(x=corpus, y=real)) + 
  geom_violin(alpha=0.5, color="gray30", fill='gray90') +
  geom_jitter(alpha=0.5, position = position_jitter(width = 0.04)) +
  xlab('') + ylab ('Probability of responding REAL') + theme_bw() 
# ggsave ('pix/real-per-sound_violin.jpg', width=9, height=6, units='cm', dpi=300, scale=2)
ggplot(temp[temp$corpus=='ut',], aes(x = real)) + geom_histogram(aes(y = ..density..), binwidth=.05, fill='gray70') + geom_density() + theme_bw()
# or:
hist (temp$real[temp$corpus=='ut'], freq=F)
lines(density(temp$real[temp$corpus=='ut'], adjust=1)) 
# NB: if some sounds were obviously real based on background noises, there would be two groups for YouTube - with very high accuracy + with ~50%. Instead, we have a nice normal distribution


## modeling per corpus
mod0 = glm (real~corpus+showEmotion, df, family='binomial')
summary(mod0)
drop1(mod0, test='Chisq')

mod = brm (real~corpus + (1|sound) + (1|id), data = df, n.warmup=100, n.iter=500, n.chains=4, n.cluster=4, family='bernoulli', ranef=F)  # ranef=T for model comparison
# summary(mod)
# stancode (mod)
# saveRDS(mod, 'mod_real~corpus.RDS')  # mod = readRDS('mod_real~corpus.RDS')
# Do we need random effects?
# mod2 = brm (real~corpus, data = df, n.warmup=100, n.iter=500, n.chains=4, n.cluster=4, family='bernoulli', ranef=T)
# LOO(mod, mod2)

plot(mod)
summary(mod)
#launch_shiny(mod)

coda = posterior_samples(mod)
colnames(coda)
coda$belin = coda[,1]
coda$cordaro = coda[,1]+coda[,2]
coda$hawk = coda[,1]+coda[,3]
coda$lima = coda[,1]+coda[,4]
coda$maurage = coda[,1]+coda[,5]
coda$simon = coda[,1]+coda[,6]
coda$ut = coda[,1]+coda[,7]

# Hawk vs other play-acted: 
antilogit = function(x) {return (1/(1+exp(-x)))}
hawk_other = antilogit(coda$hawk) - (antilogit(coda$belin)+antilogit(coda$cordaro)+antilogit(coda$lima)+antilogit(coda$maurage)+antilogit(coda$simon))/5 
quantile (hawk_other, probs=c(.5, .025, .975))

corpora = c('Belin\net al.\n(2008)','Cordaro\net al.\n(2016)','Hawk\net al.\n(2009)','Lima\net al.\n(2013)','Maurage\net al.\n(2007)','Simon-\nThomas et al.\n(2009)','Anikin &\nPersson\n(2016)')

df_plot = data.frame (corpus = c(corpora[1],corpora[2],corpora[3],corpora[4],corpora[5],corpora[6],corpora[7]), lwr=NA, fit=NA, upr=NA)
for (i in 1:nrow(df_plot)){
  df_plot[i,2:4] = as.numeric ( 1/(1+exp(-quantile (coda[,i+10], probs=c(.025,.5,.975)) )) )
}
df_plot$corpus = factor (df_plot$corpus, levels=c(corpora[7],corpora[3],corpora[2],corpora[6],corpora[4],corpora[5],corpora[1]))
df_plot[,2:4] = df_plot[,2:4] * 100 # convert to %

temp = aggregate (real~corpus+sound, df, mean)
temp$corpus=as.character(temp$corpus)
temp$corpus[1:30] = corpora[1]; temp$corpus[31:44] = corpora[2]; temp$corpus[45:58] = corpora[3]; temp$corpus[59:94] = corpora[4]; temp$corpus[95:112] = corpora[5]; temp$corpus[113:139] = corpora[6]; temp$corpus[140:nrow(temp)] = corpora[7]
temp$corpus = factor (temp$corpus, levels=c(corpora[7],corpora[3],corpora[2],corpora[6],corpora[4],corpora[5],corpora[1]))
temp[,3] = temp[,3] * 100 # convert to %


bracketsGrob <- function(...){ # see http://stackoverflow.com/questions/35633239/add-curly-braces-to-ggplot2-and-then-use-ggsave
  l <- list(...)
  e <- new.env()
  e$l <- l
  grid:::recordGrob(  {
    do.call(grid.brackets, l)
  }, e)
}

# grid.locator(unit="native")  # use to find the right coordinates in native units
myBrace = bracketsGrob(x1=0.17, y1=.8, x2=.95, y2=.8, h=.1, ticks=0.5, curvature=0.5, type=1, col='black', lwd=2, lty ="solid") # note that units here are "npc" (0 to 1)


ggplot (df_plot, aes(x=corpus, y=fit)) +
  geom_violin(data=temp, aes(x=corpus, y=real), alpha=0.5, color="gray70", fill='gray95') +
  geom_jitter(data=temp, aes(x=corpus, y=real), alpha=0.3, position = position_jitter(width = 0.07)) +
  geom_errorbar(aes(ymin=lwr, ymax=upr), position=position_dodge(), size=1, width=.2) +
  geom_point(shape=21, size=4, fill='black') +
  geom_hline(yintercept=50, linetype=2) +
  xlab("") +
  ylab('Probability of responding REAL, %') +
  annotation_custom(myBrace) + # npc units
  annotate("Text", label="Authentic", x=1, y=100, size=5) + # native units
  annotate("Text", label="Acted", x=4.4, y=100, size=5) + # native units
  theme_bw () + ylim(0,100)

# ggsave ('pix/real-per-corpus_violin_noise.jpg', width=9, height=6, scale=1.7, units='cm', dpi=300)
# ggsave ('pix/real-per-corpus_original.tiff', width=9, height=6, scale=1.7, units='cm', dpi=600)

# show outliers
a = aggregate(real~sound,df,mean)[1:139,]
a [a[,2]>.6,]

a = aggregate(real~sound,df,mean)[140:278,]
a [a[,2]<.4,]



## How does the addition of noise change the perceived authenticity of each corpus?

library(lme4)
mod = glmer (real~noisy*corpus + (1|sound) + (1|id), data = df_copy, family='binomial', nAGQ=0)
summary(mod)
drop1(mod, test='Chisq') # presence of noise does not affect overall classification accuracy

mod = brm (real~noisy*corpus + (1|sound) + (1|id), data = df_copy, n.warmup=100, n.iter=500, n.chains=4, n.cluster=4, family='bernoulli', ranef=F) 
# saveRDS (mod, 'mod_real_noisy*corpus.RDS') 
plot (mod)

coda = posterior_samples(mod)
colnames(coda)

belin_noise = antilogit (coda[,1]+coda[,2]) - antilogit (coda[,1])
quantile (belin_noise, probs=c(.5, .025, .975))

cordaro_noise = antilogit (coda[,1]+coda[,3]+coda[,2]+coda[,9]) - antilogit (coda[,1]+coda[,3])
quantile (cordaro_noise, probs=c(.5, .025, .975))

hawk_noise = antilogit (coda[,1]+coda[,4]+coda[,2]+coda[,10]) - antilogit (coda[,1]+coda[,4])
quantile (hawk_noise, probs=c(.5, .025, .975))

lima_noise = antilogit (coda[,1]+coda[,5]+coda[,2]+coda[,11]) - antilogit (coda[,1]+coda[,5])
quantile (lima_noise, probs=c(.5, .025, .975))

maurage_noise = antilogit (coda[,1]+coda[,6]+coda[,2]+coda[,12]) - antilogit (coda[,1]+coda[,6])
quantile (maurage_noise, probs=c(.5, .025, .975))

simon_noise = antilogit (coda[,1]+coda[,7]+coda[,2]+coda[,13]) - antilogit (coda[,1]+coda[,7])
quantile (simon_noise, probs=c(.5, .025, .975))

ut_noise = antilogit (coda[,1]+coda[,8]+coda[,2]+coda[,14]) - antilogit (coda[,1]+coda[,8])
quantile (ut_noise, probs=c(.5, .025, .975))



