setwd ('/home/allgoodguys/Documents/Studying/Lund_PhD/experiments/real-fake/analysis')
require(brms)
require(shinystan)
require(ggplot2)

df_copy = read.csv('real-fake_indiv-answers.csv')
# df = droplevels (df_copy[df_copy$noisy==T,])  # noisy only 
# df = droplevels (df_copy[df_copy$noisy!=T,])  # w/o noise only 

## Which emotions >likely to be rated as real in ut vs other corpora?
aggregate (real~ut+emotion, df, mean)

temp = aggregate (real~emotion+ut+sound, df, mean)
boxplot (real~ut+emotion, temp)

mod0 = glm (real~ut+emotion, df, family='binomial')
summary(mod0)
drop1(mod0, test='Chisq')

mod = brm (real~ut*emotion + (1|sound) + (1|id), data = df, n.warmup=100, n.iter=500, n.chains=4, n.cluster=4, family='bernoulli', ranef=F)  # 
# saveRDS(mod, 'mod_real~ut-emotion_noise.RDS')

plot(mod)
summary(mod)
# launch_shiny(mod)

coda = posterior_samples(mod)
colnames(coda)

a = data.frame(achivement_other=coda[,1], 
               achievement_ut=coda[,1]+coda[,2],
               amusement_other=coda[,1]+coda[,3], 
               amusement_ut=coda[,1]+coda[,2]+coda[,3]+coda[,10],
               anger_other=coda[,1]+coda[,4], 
               anger_ut=coda[,1]+coda[,2]+coda[,4]+coda[,11],
               disgust_other=coda[,1]+coda[,5], 
               disgust_ut=coda[,1]+coda[,2]+coda[,5]+coda[,12],
               fear_other=coda[,1]+coda[,6], 
               fear_ut=coda[,1]+coda[,2]+coda[,6]+coda[,13],
               pain_other=coda[,1]+coda[,7], 
               pain_ut=coda[,1]+coda[,2]+coda[,7]+coda[,14],
               pleasure_other=coda[,1]+coda[,8], 
               pleasure_ut=coda[,1]+coda[,2]+coda[,8]+coda[,15],
               sadness_other=coda[,1]+coda[,9], 
               sadness_ut=coda[,1]+coda[,2]+coda[,9]+coda[,16] )
a = apply (a, 2, function(x)1/(1+exp(-x)))


df_plot = data.frame (emotion = rep(c('Achievement','Amusement','Anger','Disgust','Fear','Pain','Pleasure','Sadness'),each=2), corpus=rep(c('Acted','Authentic'),8), lwr=NA, fit=NA, upr=NA)
df_plot[,3:5] = t ( apply (a, 2, function(x)quantile(x,probs=c(.025,.5,.975))) )
df_plot$corpus = factor (df_plot$corpus, levels=c('Authentic','Acted'))


### contrasts
df_contrasts = data.frame (emotion = levels(df$emotion), lwr=NA, fit=NA, upr=NA)
for (i in 1:length(levels(df$emotion))) {
  df_contrasts[i,2:4] = quantile (a[,2*i]-a[,2*i-1], probs=c(.025,.5,.975))
}
contrasts = c ( paste0( round(df_contrasts$fit*100,0), ' [', round(df_contrasts$lwr*100,0), ', ', round(df_contrasts$upr*100,0), ']') )

# to add contrasts to the plot, re-label factor levels in the dataframe
df_plot$emotion = as.character(df_plot$emotion)
for (i in 1:length(unique(df_plot$emotion))){
  df_plot$emotion[2*i] = paste0(df_plot$emotion[2*i],'\n',contrasts[i]) # \n for new line - looks good in ggplot
  df_plot$emotion[2*i-1] = paste0(df_plot$emotion[2*i-1],'\n',contrasts[i])
}
df_plot$emotion = as.factor(df_plot$emotion)
df_plot$fit = df_plot$fit * 100 # convert to %
df_plot$lwr = df_plot$lwr * 100
df_plot$upr = df_plot$upr * 100

ggplot (df_plot, aes(x=emotion, y=fit, fill=corpus)) +
  geom_bar(stat='identity', position=position_dodge()) + 
  geom_errorbar(aes(ymin=lwr, ymax=upr), position=position_dodge()) +
  geom_hline(yintercept=50, linetype=2) +
  scale_fill_manual(values=c('gray75','gray40'), name='Corpus', breaks=c('Authentic','Acted')) +
  xlab("") +
  ylab('Probability of responding REAL, %') +
  theme_bw ()

# ggsave ('pix/real-per-emotion.jpg', width=15, height=6, units='cm', dpi=300, scale=2)


## plotting with violins
temp = aggregate (real~emotion+sound, df, mean)
temp$corpus = rep(c('Acted','Authentic'),each=139)
temp$corpus = factor(temp$corpus, levels=c('Authentic','Acted'))
temp$emotion = as.character(temp$emotion)
temp$emotion = df_plot$emotion[match(substr(temp$emotion,2,4),substr(df_plot$emotion,2,4))] # otherwise the plot doesn't work - factor levels should be the same for both layers
temp$real = temp$real * 100 # convert to %

ggplot (df_plot, aes(x=emotion, y=fit, fill=corpus)) +
  geom_violin(data=temp, aes(x=emotion, y=real, fill=corpus, col='white'), alpha=0.3, position=position_dodge(.7)) +
  geom_point(data=temp, aes(x=emotion, y=real), alpha=.5, position = position_jitterdodge(dodge.width=.7, jitter.width=.1), show.legend=FALSE) +
  scale_color_manual(values=c('gray75','gray40')) +
  guides(col=FALSE) +
  geom_errorbar(aes(ymin=lwr, ymax=upr), position=position_dodge(.7), width=.3, size=1) +
  geom_point(shape=21, size=4, position=position_dodge(.7), show.legend=FALSE) +
  geom_hline(yintercept=50, linetype=2) +
  scale_fill_manual(values=c('gray85','gray50'), name='', breaks=c('Authentic','Acted')) +
  xlab("") +
  ylab('Probability of responding REAL, %') + expand_limits(y=c(0,100)) +
  theme_bw () + ylim(0,100)

# ggsave ('pix/real-per-emotion_violin_noise.jpg', width=15, height=6, units='cm', dpi=300, scale=2)
# ggsave ('pix/real-per-emotion_masking-noise.tiff', width=15, height=6, units='cm', dpi=600, scale=2)

### is overall accuracy better than chance? Table 2
# pooled
df = df_copy
library(lme4)
mod = glmer (correct~noisy + (1|sound) + (1|id), data = df, family='binomial', nAGQ=0)
summary(mod)
drop1(mod, test='Chisq') # presence of noise does not affect overall classification accuracy

mod = glmer (real~noisy + (1|sound) + (1|id), data = df, family='binomial', nAGQ=0)
summary(mod)
drop1(mod, test='Chisq') 

mod = glmer (correct~emotion*noisy + (1|sound) + (1|id), data = df, family='binomial', nAGQ=0)
summary(mod)
drop1(mod, test='Chisq') # no interaction between noise and emotion

mod = glmer (real~emotion*noisy + (1|sound) + (1|id), data = df, family='binomial', nAGQ=0)
summary(mod)
drop1(mod, test='Chisq') # no interaction between noise and emotion

# with noise
df = droplevels (df_copy[df_copy$noisy==T,])
mod = brm (correct~emotion + (1|sound) + (1|id), data = df, n.warmup=100, n.iter=500, n.chains=4, n.cluster=4, family='bernoulli', ranef=F)
mod_total = brm (correct~1 + (1|sound) + (1|id), data = df, n.warmup=100, n.iter=500, n.chains=4, n.cluster=4, family='bernoulli', ranef=F)

coda = posterior_samples(mod)
coda_total = posterior_samples(mod_total)
total = apply (as.matrix(coda_total[,1]), 1, function(x)1/(1+exp(-x)))
total = quantile(total, probs=c(.5, .025, .975))
total = round(total*100, 1)
total_noise = paste0(total[1], ' [', total[2], ', ', total[3], ']')

# colnames(coda)
a = data.frame(achivement=coda[,1], amusement=coda[,1]+coda[,2], 
               anger=coda[,1]+coda[,3], disgust=coda[,1]+coda[,4], 
               fear=coda[,1]+coda[,5], pain=coda[,1]+coda[,6], 
               pleasure=coda[,1]+coda[,7], sadness=coda[,1]+coda[,8])
a = apply (a, 2, function(x)1/(1+exp(-x)))
table_noise = t ( as.matrix ( apply (a, 2, function(x)quantile(x,probs=c(.5, .025, .975))) ) )
table_noise = round (table_noise*100, 1)

# without noise
df = droplevels (df_copy[df_copy$noisy!=T,])
mod = brm (correct~emotion + (1|sound) + (1|id), data = df, n.warmup=100, n.iter=500, n.chains=4, n.cluster=4, family='bernoulli', ranef=F)
mod_total = brm (correct~1 + (1|sound) + (1|id), data = df, n.warmup=100, n.iter=500, n.chains=4, n.cluster=4, family='bernoulli', ranef=F)

coda = posterior_samples(mod)
coda_total = posterior_samples(mod_total)
total = apply (as.matrix(coda_total[,1]), 1, function(x)1/(1+exp(-x)))
total = quantile(total, probs=c(.5, .025, .975))
total_noNoise = paste0(round(total[1]*100,1), ' [', round(total[2]*100,1), ', ', round(total[3]*100,1), ']')

a = data.frame(achivement=coda[,1], amusement=coda[,1]+coda[,2], 
               anger=coda[,1]+coda[,3], disgust=coda[,1]+coda[,4], 
               fear=coda[,1]+coda[,5], pain=coda[,1]+coda[,6], 
               pleasure=coda[,1]+coda[,7], sadness=coda[,1]+coda[,8])
a = apply (a, 2, function(x)1/(1+exp(-x)))
table_noNoise = t ( as.matrix ( apply (a, 2, function(x)quantile(x,probs=c(.5, .025, .975))) ) )
table_noNoise = round (table_noNoise*100, 1)

table_short = data.frame (emotion=c (rownames(table_noise),'Total'),
                          noNoise = c (apply(table_noNoise, 1, function(x)paste0(x[1], ' [', x[2], ', ', x[3], ']')), total_noNoise),
                          noise = c (apply(table_noise, 1, function(x)paste0(x[1], ' [', x[2], ', ', x[3], ']')), total_noise ) )
# write.csv (table_short, '~/Downloads/temp.csv')




## How does the addition of noise change the perceived authenticity of each emotion?
library(lme4)
mod = glmer (real~noisy*emotion + (1|sound) + (1|id), data = df_copy, family='binomial', nAGQ=0)
summary(mod)
drop1(mod, test='Chisq') # presence of noise does not affect overall classification accuracy

mod = brm (real~noisy*emotion + (1|sound) + (1|id), data = df_copy, n.warmup=100, n.iter=500, n.chains=4, n.cluster=4, family='bernoulli', ranef=F) 
# saveRDS (mod, 'mod_real_noisy*emotion.RDS') 
plot (mod)

coda = posterior_samples(mod)
colnames(coda)

achievement_noise = antilogit (coda[,1]+coda[,2]) - antilogit (coda[,1])
quantile (achievement_noise, probs=c(.5, .025, .975))

amusement_noise = antilogit (coda[,1]+coda[,3]+coda[,2]+coda[,10]) - antilogit (coda[,1]+coda[,3])
quantile (amusement_noise, probs=c(.5, .025, .975))

anger_noise = antilogit (coda[,1]+coda[,4]+coda[,2]+coda[,11]) - antilogit (coda[,1]+coda[,4])
quantile (anger_noise, probs=c(.5, .025, .975))

disgust_noise = antilogit (coda[,1]+coda[,5]+coda[,2]+coda[,12]) - antilogit (coda[,1]+coda[,5])
quantile (disgust_noise, probs=c(.5, .025, .975))

fear_noise = antilogit (coda[,1]+coda[,6]+coda[,2]+coda[,13]) - antilogit (coda[,1]+coda[,6])
quantile (fear_noise, probs=c(.5, .025, .975))

pain_noise = antilogit (coda[,1]+coda[,7]+coda[,2]+coda[,14]) - antilogit (coda[,1]+coda[,7])
quantile (pain_noise, probs=c(.5, .025, .975))

pleasure_noise = antilogit (coda[,1]+coda[,8]+coda[,2]+coda[,15]) - antilogit (coda[,1]+coda[,8])
quantile (pleasure_noise, probs=c(.5, .025, .975))

sadness_noise = antilogit (coda[,1]+coda[,9]+coda[,2]+coda[,16]) - antilogit (coda[,1]+coda[,9])
quantile (sadness_noise, probs=c(.5, .025, .975))

