setwd ('/home/allgoodguys/Documents/Studying/Lund_PhD/experiments/real-fake/analysis')
require(brms)
require(shinystan)
require(ggplot2)
library(grid)
library(pBrackets) 

df = read.csv('real-fake_indiv-answers.csv')

aggregate (correct~langSame, df, mean)  # no effect at all
aggregate (correct~langSame+corpus, df, mean)
aggregate (real~langSame+corpus, df, mean)
aggregate (real~langSame + (df$corpus=='ut'), df, mean)

# to create a nice cross-table
temp = aggregate (real~langSame+corpus, df, mean)
xtabs (real~., data=temp)

temp = aggregate (real~langRater+langCaller, df, mean)
# table (df$langRater, df$langCaller)
xtabs (real~., data=temp)


## model
mod0 = brm (correct~langSame + (1|sound) + (1|id), data = df, n.warmup=100, n.iter=500, n.chains=4, n.cluster=4, family='bernoulli', ranef=F)
plot(mod0)
a = summary(mod0) # exp(c(-0.04, -0.14, 0.05))  
exp (a$fixed[2,c(1,3,4)])

mod0 = glm (real~langSame * corpus , df, family='binomial')
summary(mod0)
drop1(mod0, test='Chisq')

library(lme4)
mod0 = glmer (real~langRater*langCaller + (1|sound) + (1|id), df, family='binomial', nAGQ = 0)
summary(mod0)
drop1(mod0, test='Chisq')



mod = brm (real~langSame * corpus + (1|sound) + (1|id), data = df, n.warmup=100, n.iter=500, n.chains=4, n.cluster=4, family='bernoulli', ranef=F)
# saveRDS (mod,'mod_real~langSame.RDS')  # mod = readRDS('mod_real~langSame.RDS')
plot(mod)
summary(mod)

coda = posterior_samples(mod)
colnames(coda)

a = data.frame (langDif_corpusBelin = coda[,1],
                langSame_corpusBelin = coda[,1]+coda[2],
                langDif_corpusCordaro = coda[,1]+coda[,3],
                langSame_corpusCordaro = coda[,1]+coda[,2]+coda[,3]+coda[,9],
                langDif_corpusHawk = coda[,1]+coda[,4],
                langSame_corpusHawk = coda[,1]+coda[,2]+coda[,4]+coda[,10],
                langDif_corpusLima = coda[,1]+coda[,5],
                langSame_corpusLima = coda[,1]+coda[,2]+coda[,5]+coda[,11],
                langDif_corpusMaurage = coda[,1]+coda[,6],
                langSame_corpusMaurage = coda[,1]+coda[,2]+coda[,6]+coda[,12],
                langDif_corpusSimon = coda[,1]+coda[,7],
                langSame_corpusSimon = coda[,1]+coda[,2]+coda[,7]+coda[,13],
                langDif_corpusYouTube = coda[,1]+coda[,8],
                langSame_corpusYouTube = coda[,1]+coda[,2]+coda[,8]+coda[,14])
a = apply (a, 2, function(x)1/(1+exp(-x)))

corpora = c('Belin\net al.\n(2008)','Cordaro\net al.\n(2016)','Hawk\net al.\n(2009)','Lima\net al.\n(2013)','Maurage\net al.\n(2007)','Simon-\nThomas et al.\n(2009)','Anikin &\nPersson\n(2016)')

df_plot = data.frame (corpus = rep(c(corpora[1],corpora[2],corpora[3],corpora[4],corpora[5],corpora[6],corpora[7]),each=2), respondent=rep(c('Different','Same'),7), lwr=NA, fit=NA, upr=NA)
df_plot[,3:5] = t ( apply (a, 2, function(x)quantile(x,probs=c(.025,.5,.975))) )

# contrasts
df_contrasts = data.frame (corpus = c(corpora[1],corpora[2],corpora[3],corpora[4],corpora[5],corpora[6],corpora[7]), lwr=NA, fit=NA, upr=NA)
for (i in 1:length(levels(df_plot$corpus))) {
  df_contrasts[i,2:4] = quantile (a[,2*i]-a[,2*i-1], probs=c(.025,.5,.975))
}
contrasts = c ( paste0( round(df_contrasts$fit*100,0), ' [', round(df_contrasts$lwr*100,0), ', ', round(df_contrasts$upr*100,0), ']') )

# to add contrasts to the plot, re-label factor levels in the dataframe
df_plot$corpus = as.character(df_plot$corpus)
for (i in 1:length(unique(df_plot$corpus))){
  df_plot$corpus[2*i] = paste0(df_plot$corpus[2*i],'\n',contrasts[i]) # \n for new line - looks good in ggplot
  df_plot$corpus[2*i-1] = paste0(df_plot$corpus[2*i-1],'\n',contrasts[i])
}
ll = unique(df_plot$corpus)
df_plot$corpus = factor(df_plot$corpus, levels=c(ll[7], ll[3], ll[2], ll[6], ll[4], ll[5], ll[1]))

# remove bars with no data
# df_plot[6,3:5] = NA
bracketsGrob <- function(...){ # see http://stackoverflow.com/questions/35633239/add-curly-braces-to-ggplot2-and-then-use-ggsave
  l <- list(...)
  e <- new.env()
  e$l <- l
  grid:::recordGrob(  {
    do.call(grid.brackets, l)
  }, e)
}

myBrace = bracketsGrob(x1=0.18, y1=.8, x2=.96, y2=.8, h=.1, ticks=0.5, curvature=0.5, type=1, col='black', lwd=2, lty ="solid") # note that units here are "npc" (0 to 1)

ggplot (df_plot, aes(x=corpus, y=fit, fill=respondent)) +
  geom_bar(stat='identity', position=position_dodge(.6), width=.6) + 
  geom_errorbar(aes(ymin=lwr, ymax=upr), position=position_dodge(.6), width=.2) +
  geom_hline(yintercept=.5, linetype=2) +
  scale_fill_manual(values=c('gray75','gray40'), name="Rater's L1", labels=c('Different','Same')) +
  xlab("") +
  ylab('Probability of responding REAL, %') +
  annotation_custom(myBrace) + # npc units
  annotate("Text", label="Authentic", x=1.1, y=1, size=5) + # native units
  annotate("Text", label="Acted", x=4.5, y=1, size=5) + # native units
  theme_bw () +
  theme(legend.position="top")


# ggsave ('pix/real-per-L1-of-actor-and-respondent.jpg', width=9, height=7, units='cm', dpi=300, scale=1.5)
# ggsave ('pix/real-per-L1-of-actor-and-respondent.tiff', width=9, height=7, units='cm', dpi=600, scale=1.5)
