## For Table 3.2.1 - acoustic descriptives per corpus. With Cohen's d
# Ideas: re-write the JAGS model per variable rather than per emotion! ~ANOVA

library(rstan); library(shinystan); library (brms)
source("~/Documents/Studying/Lund_PhD/epistles/02_article_triplets/triplets_analysis/source_for_multiple_cores.R")

setwd('/home/allgoodguys/Documents/Studying/Lund_PhD/epistles/04_article_nat-actor/analysis')


## load data
all = read.csv ('all_july2016.csv')
all [,c('specSlope_median','specSlope_sd')] = all [,c('specSlope_median','specSlope_sd')] * 1000 # convert to kHz
all [,c('specSlope_median','specSlope_sd','ampl_voiced_median','ampl_voiced_sd','voiced')] = 100*all [,c('specSlope_median','specSlope_sd','ampl_voiced_median','ampl_voiced_sd','voiced')] # convert to %
all = droplevels (subset(all, all$emotion!='joy' & all$emotion!='effort' & all$emotion!='achievement'))
# aggregate (duration~corpus, all, mean)
# aggregate (duration~corpus, all, sd)

all$corpus = as.factor ( ifelse (all$corpus=='ut','YouTube','Other') )
corpora = c('YouTube','Other') # or corpora = levels(all$corpus)
for (c in corpora){
  assign(c, droplevels (subset(all, all$corpus==c)))
}

## specify the acoustic variables to analyze and the required precision (digits after .)
vars = c("duration","pitch_manually", "pitch_sd", "HNR_median","HNR_sd", "quartile25_median", "quartile25_sd", "harmonics_median",  "harmonics_sd", "specSlope_median", "specSlope_sd", "syllableLength_median", "syllableLength_sd", "interBurst_median", "interBurst_sd", "ampl_voiced_median", "ampl_voiced_sd", "voiced")
precision = c (2,0,0,1,1,0,0,1,1,1,1,0,0,0,0,0,0,0)

# for fewer vars (only good predictors of emotion):
# vars = c("pitch_manually", "HNR_median","bandLwr_median", "interBurst_median","ampl_RMS_median")
# precision = c (0,2,0,0,2)

## table of acoustic descriptives
result = matrix (nrow=length(vars)*length(corpora), ncol=length(levels(all$emotion))) 
colnames(result) = levels(all$emotion)
rownames(result) = rep (vars, each=length(corpora)) 

for (v in 1:length(vars)) {
  for (em in 1:length(levels(all$emotion))) {
    for (c in 1:length(corpora)){
      temp = get(corpora[c])
      assign (paste0('mean_',corpora[c]), try(aggregate (temp[,vars[v]]~emotion, data=temp, FUN=mean), silent=T))
      assign (paste0('sd_',corpora[c]), try(aggregate (temp[,vars[v]]~emotion, data=temp, FUN=sd), silent=T))
      r = length(corpora)*v - length(corpora) + c  # row number in the output
      temp_mean = get(paste0('mean_',corpora[c]))
      temp_sd = get(paste0('sd_',corpora[c]))
      result[r,em] = paste0 ( round (as.numeric(temp_mean[temp_mean['emotion']==levels(all$emotion)[em]][2]), precision[v]), 
                              ' ± ', 
                              round (as.numeric(temp_sd [temp_sd['emotion']==levels(all$emotion)[em]][2]), precision[v]) )
      # and clear up the mess
      rm ( list = c(paste0('mean_',corpora[c]), paste0('sd_',corpora[c]))) 
    }
    rm (list = c('temp','r','temp_mean','temp_sd'))
  }
}

result[result=='NA ± NA'] <- '-'   # replace NA by '-'
corpus = rep ( corpora, length(vars) )
result = cbind (corpus, result)
# write.csv (result, '~/Downloads/temp.csv') 

## Cohen's d
m1 = as.numeric ( apply ( result[seq(1, (nrow(result)-1), by=2),2:8], c(1,2), function(x) unlist (strsplit (x, ' ± ') )[1] ) )
m2 = as.numeric ( apply ( result[seq(2, nrow(result), by=2),2:8], c(1,2), function(x) unlist (strsplit (x, ' ± ') )[1] ) )
m1-m2

s1 = as.numeric ( apply ( result[seq(1, (nrow(result)-1), by=2),2:8], c(1,2), function(x) unlist (strsplit (x, ' ± ') )[2] ) )^2
s2 = as.numeric ( apply ( result[seq(2, nrow(result), by=2),2:8], c(1,2), function(x) unlist (strsplit (x, ' ± ') )[2] ) )^2
n1 = rep (table(YouTube$emotion), each=nrow(result)/2)
n2 = rep (table(Other$emotion), each=nrow(result)/2)
d = round ( (m1-m2) / sqrt( ((n1-1)*s1 + (n2-1)*s2)/(n1+n2-2) ), 2)

result1 = matrix (ncol=ncol(result), nrow=(nrow(result)/2*3) ) 
colnames(result1) = colnames(result)
rownames(result1) = rep(vars, each=3)
row_count = 1
d_count = 1
for (i in 1:nrow(result1)){
  if (i/3 == i %/% 3) {
    result1[i, 2:ncol(result1)] = d[seq(d_count, length(d), by=nrow(result1)/3)]
    d_count = d_count + 1
  } else {
    result1[i, 2:ncol(result1)] = result[row_count, 2:ncol(result)]
    row_count = row_count + 1
  } 
}
result1[,'corpus'] = rep(c("YouTube","Other","Cohen's d"))

# average d per emotion (absolute value): 
av_d_per_em = apply (result1[result1[,'corpus']=="Cohen's d" , 2:8], 2, function(x)mean(abs(as.numeric(x)),na.rm=T))
result1 = rbind (result1, c('d_per_em',round(av_d_per_em,2)))
# average d per variable:
av_d_per_var = round ( apply (result1[result1[,'corpus']=="Cohen's d" , 2:8], 1, function(x)mean(as.numeric(x),na.rm=T)), 2 )
mean (av_d_per_em)
result1 = as.data.frame(result1)
result1$av_d_per_var = rep(NA, nrow(result1))
result1$av_d_per_var[seq(3, nrow(result1)-1, by=3)] = av_d_per_var

# write.csv (result1, '~/Downloads/temp.csv') 



## STAN stuff
rescale_df = function(df){
  df$pitch_manually = ifelse (!is.na(df$pitch_manually), log2(df$pitch_manually), NA)
  df$pitch_sd = log2(df$pitch_sd+1) # b/c it's occasionally 0
  df$quartile25_median = log2(df$quartile25_median+1)
  df$quartile25_sd = log2(df$quartile25_sd+1)
  return (df)
}

all = rescale_df (all)

modelString = "
data { 
int<lower=1> N;  # number of observations 
int Y[N];  # response variable 
int<lower=1> K;  # number of fixed effects 
matrix[N, K] X;  # FE design matrix 
} 
transformed data { 
} 
parameters { 
real b_Intercept;  # fixed effects Intercept 
vector[K] b;  # fixed effects 
real<lower=0.01> sigmaX; # SD of betas
} 
transformed parameters { 
vector[N] eta;  # linear predictor 
# compute linear predictor 
eta = X * b + b_Intercept; 
} 
model { 
# prior specifications 
sigmaX ~ normal (0, 10);
b ~ cauchy(0, sigmaX);
# likelihood contribution 
Y ~ bernoulli_logit(eta); 
} 
"
stanmod <- stan_model(model_code = modelString, model_name = "stanmod") 


## stan model for all emotions together
vars = c("pitch_manually", "pitch_sd", "HNR_median","HNR_sd", "quartile25_median", "quartile25_sd", "harmonics_median", "specSlope_median", "specSlope_sd", "syllableLength_median", "syllableLength_sd", "interBurst_median", "interBurst_sd", "ampl_voiced_median", "ampl_voiced_sd", "voiced") # exclude harmonics_sd (it depends too much on correct pitch tracking, so it's misleading)
df = all[, c("corpus","emotion",vars)]
df[,3:ncol(df)] = scale (df[,3:ncol(df)]) # scale all vars b/c 1) it's needed for the model, and 2) it's a must if we remove NA's by making them zeroes - for scaled variables this will correspond to a "typical" value rather than an outrageous outlier
df$authentic = ifelse(df$corpus=='YouTube',1,0)
formula = paste0('authentic ~ ',paste(vars,collapse='+')) # prepare the formula for STAN

# deal with NA's. Either exlude: df = na.omit (df)
# or make into zeroes (NB: on a z-scale!):
df[is.na(df)] <- 0
# all$pitch_median = log2(all$pitch_median)
# df$pitchCorrection = all$pitch_manually-all$pitch_median; df[is.na(df)] <- 0
# write.csv ( cor(df[3:ncol(df)]), '~/Downloads/temp.csv')

dataList = make_standata (as.formula(formula), data = df)
fit <- sampling(stanmod, data = dataList, pars = c( "b_Intercept" ,  "b", 'sigmaX' ), chains = 4, cores = 4, iter = 5000, thin = 1)
# saveRDS(fit, 'mod_authentic-resp~acoustics.RDS')  # fit = readRDS('mod_authentic-resp~acoustics.RDS')
# fit <- stan(model_code=modelString, data = dataList, pars = c( 'b_Intercept' ,  'b', 'sigmaX' ), chains = 4, iter = 5000, thin = 1)


stan_trace(fit)
a = as.matrix(fit)
b = stan_plot (fit, pars = colnames(a)[2:(ncol(a)-2)]) # + theme(panel.grid=element_line(size=2, colour='gray50', linetype=1)) # doesn't work for some reason...
# b$scales$scales[[2]]$labels = vars
b$scales$scales[[2]]$labels = c('Pitch median','Pitch SD','HNR median','HNR SD','First quartile median','First quartile SD','Energy above F0 median','Spectral slope median','Spectral slope SD', 'Syllable median','Syllable SD','Interburst median','Interburst SD','Amplitude median','Amplitude SD','Voiced')  # NB: set the labels in ggplot object to the variable names, otherwise you get b[1], b[2] etc. Clever, no?  :))
b + geom_vline(xintercept=0, linetype=2) + theme(panel.grid=element_line(size=2, colour='gray50', linetype=1))     # + ggtitle("All emotions")
# ggsave ('/home/allgoodguys/Documents/Studying/Lund_PhD/epistles/04_article_nat-actor/pix/betas_00-pooled.jpg', width=8, height=10, scale=2, units='cm', dpi=300)
# ggsave ('/home/allgoodguys/Documents/Studying/Lund_PhD/epistles/04_article_nat-actor/pix/betas_00-pooled.tiff', width=8, height=10, scale=2, units='cm', dpi=600)

# a quick sanity check for subgroups of vars
summary(glm(authentic~pitch_manually+pitch_sd+HNR_median+HNR_sd+quartile25_median+quartile25_sd+harmonics_median+specSlope_median+specSlope_sd+syllableLength_median+syllableLength_sd+interBurst_median+interBurst_sd+ampl_voiced_median+ampl_voiced_sd+voiced, df, family='binomial'))

summary(glm(authentic~pitch_manually+pitch_sd+HNR_median+harmonics_sd+specSlope_median+specSlope_sd+syllableLength_median+syllableLength_sd+interBurst_median+ampl_voiced_sd+voiced, df, family='binomial'))
# pitch_sd becomes negative once we consider harmonics_sd. 
plot(df$pitch_sd, df$harmonics_sd, col=as.numeric(as.factor(df$authentic))+2)
boxplot (harmonics_sd~authentic, df)
boxplot (pitch_sd~authentic, df)


# check model accuracy:
colnames(a)=c('int',vars,'sigmaX','lp__')
coefs = apply (a, 2, median)
for (i in 1:nrow(df)){
  logit = coefs[1] + sum(coefs[vars]*df[i,vars])
  df$authenticity[i] = 1/(1+exp(-logit))
}
df$pred = ifelse (df$authenticity>.5, 'YouTube','Other')
df$correct = (df$pred==df$corpus)
mean(df$correct)
aggregate (correct~corpus, df, mean)
aggregate (correct~emotion, df, mean)

df$authentic = as.factor(df$authentic)
library(randomForest)
mod3 = randomForest (authentic ~ pitch_manually+pitch_sd+HNR_median+HNR_sd+quartile25_median+quartile25_sd+harmonics_median+harmonics_sd+specSlope_median+specSlope_sd+syllableLength_median+syllableLength_sd+interBurst_median+interBurst_sd+ampl_voiced_median+ampl_voiced_sd+voiced, df, importance=TRUE, ntree=2000, strata=df$authentic, sampsize=rep(min(table(df$authentic)), length(unique(df$authentic))))
mod3 # 83% correct w/o emotion, 85% with emotion
varImpPlot(mod3, type='1')

# w/o harmonics_sd
mod4 = randomForest (authentic ~ pitch_manually+pitch_sd+HNR_median+HNR_sd+quartile25_median+quartile25_sd+harmonics_median+specSlope_median+specSlope_sd+syllableLength_median+syllableLength_sd+interBurst_median+interBurst_sd+ampl_voiced_median+ampl_voiced_sd+voiced+emotion, df, importance=TRUE, ntree=2000, strata=df$authentic, sampsize=rep(min(table(df$authentic)), length(unique(df$authentic))))
mod4 # 78% correct w/o emotion, 80% with emotion
varImpPlot(mod4, type='1')

mod5 = randomForest (authentic ~ harmonics_sd, df, importance=TRUE, ntree=2000, strata=df$authentic, sampsize=rep(min(table(df$authentic)), length(unique(df$authentic))))
mod5 # 70% correct 
varImpPlot(mod5, type='1')


YouTube = rescale_df(YouTube)
YouTube = YouTube[, c("emotion",'quartile25_median' ,'pitch_manually' ,'HNR_median' ,'ampl_voiced_median' ,'interBurst_median')]
YouTube[,2:ncol(YouTube)] = scale (YouTube[,2:ncol(YouTube)]) 
YouTube[is.na(YouTube)] = 0
mod6_em = randomForest (emotion ~ quartile25_median + pitch_manually + HNR_median + ampl_voiced_median + interBurst_median, YouTube, ntree=5000, strata=YouTube$emotion, sampsize=rep(min(table(YouTube$emotion)), length(unique(YouTube$emotion))))
mod6_em # 57% correct
varImpPlot(mod6_em)

mod6 = randomForest (authentic ~ quartile25_median + pitch_manually + HNR_median + ampl_voiced_median + interBurst_median, df, ntree=5000, strata=df$authentic, sampsize=rep(min(table(df$authentic)), length(unique(df$authentic))))
mod6 # 72% correct
varImpPlot(mod6)




### Check that pitch is really higher in authentic vocs vs. actor portrayals, controlling for gender
all$male = ifelse (all$sex=='m', T, F)
all$ut = ifelse (all$corpus=='ut', T, F)
all = rescale_df (all)
all$pitch_manually_scaled = scale(all$pitch_manually)

mod = lm (pitch_manually_scaled ~ male*ut, all)
summary(mod)
drop1(mod, test='F')

mod1 = lm (pitch_manually_scaled ~ male+ut, all)
summary(mod1)
drop1(mod1, test='F')

mod2 = lm (pitch_manually_scaled ~ male+ut*emotion, all)
summary(mod2)
drop1(mod2, test='F')
